"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _classStaticPrivateMethodGet;
var _assertClassBrand = require("./assertClassBrand.js");
function _classStaticPrivateMethodGet(receiver, classConstructor, method) {
  (0, _assertClassBrand.default)(classConstructor, receiver);
  return method;
}

//# sourceMappingURL=classStaticPrivateMethodGet.js.map
