"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _classCheckPrivateStaticAccess;
var _assertClassBrand = require("assertClassBrand");
function _classCheckPrivateStaticAccess(receiver, classConstructor, returnValue) {
  return _assertClassBrand(classConstructor, receiver, returnValue);
}

//# sourceMappingURL=classCheckPrivateStaticAccess.js.map
