"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _writeOnlyError;
function _writeOnlyError(name) {
  throw new TypeError('"' + name + '" is write-only');
}

//# sourceMappingURL=writeOnlyError.js.map
