"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = readInputSourceMapFile;
function readInputSourceMapFile() {
  throw new Error("Reading input source map files is not supported in browsers");
}
0 && 0;

//# sourceMappingURL=read-input-source-map-file-browser.js.map
