"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _nullishReceiverError;
function _nullishReceiverError(r) {
  throw new TypeError("Cannot set property of null or undefined.");
}

//# sourceMappingURL=nullishReceiverError.js.map
